/*
 *
 *
 *    Resea (x86/api.c)
 *    Mafice <mafice@mflabo.net>
 *
 *
 */

#include "kernel.h"



/*
*  Function: APIEntry
*
*    APIエントリ
*
*  Parameters:
*
*    eax -  EAXレジスタ
*    ebx -  EBXレジスタ
*    ecx -  ECXレジスタ
*    edx -  EDXレジスタ
*    edi -  EDIレジスタ
*    esi -  ESIレジスタ
*
*  Returns:
*
*    各APIに依存します。
*
*/

u32 APIEntry (u32 eax, u32 ebx, u32 ecx, u32 edx, u32 edi, u32 esi){



  //putchar
  if(eax == API_putchar){

    printchar((char) edi, Screen.textColor|Screen.bgColor);
    return 0;
  }


  //out
  if(eax == API_out){

    if(ecx == 8)
      asm_out8((u16) edi, (u8) ebx);

    return 0;
  }


  //in
  if(eax == API_in){

    if(ecx == 8)
      return asm_in8((u16) esi);
  }


  //setISR
  if(eax == API_setISR){

    if((0 <= esi) && (esi <= ISR_MAX)){

      Interrupt.isr[esi] = edx;
      return true;

    }else{

      return false;
    }
  }


  //waitInterrupt
  if(eax == API_waitInterrupt){

    asm_sti();
    asm_hlt();
    return 0;
  }


  return 0;
}

